/*
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _PROTOTYPE_H_
#define _PROTOTYPE_H_

#include <sys/types.h>
#include "typedef.h"
#include "have_config.h"

/* main.c */
int decode_getopt (int argc, char **argv);
int xstrtol (const char *str);
int check_ip (void);
int parse_tab_ip (const char *ip, int *tab);
int thread_config (void);
int start_thread (pthread_attr_t *attr);
void * fun_thread (void *argu);
void free_one_argv_thread (const int index);
int get_thread_index (void);
int incremente_ip (char *ip);
void push_pile (const char *ip, const char *perm);
void free_pile (void);
void signal_init (void);
void signal_handler (int sig);
void write_log (const char *ip, const char *perm);
/****/

/* exit.c */
void exit_program (int code);
/****/

/* memory.c */
void * xmalloc (const int size);
void * xcalloc (const int size);
char * xstrdup (const char *str);
void xfree (void **ptr);
void clean_memory (void);
/****/

/* print.c */
void print_version (void);
void print_help (void);
void print_verbose_argv (void);
void print_started (void);
void print_static (void);
void print_static_current (void);
/****/

/* socket.c */
struct socket_s * create_socket (const char *ip, const int port);
int create_socket_stream (void);
int connect_socket (struct socket_s *sock);
int block_socket (int socket);
int unblock_socket (int socket);
void free_socket (struct socket_s **socket);
#ifdef HAVE_SSLLIB
SSL * connect_ssl (int socket);
void free_ssl (struct socket_s **socket);
#endif /* HAVE_SSLLIB */
/****/

/* ftp.c */
int nsa_ftp_client (struct socket_s *socket, char **permission);
int nsa_ftp_connect (struct socket_s *socket);
void nsa_get_ftp_permission (struct socket_s *socket_ftp, char **permission);
struct socket_s * nsa_ftp_creat_pasv_socket (struct socket_s *socket_ftp);
void get_pasiv_data (const char *info_serv, struct pasv_s *data);
/****/

/* pop3.c */
int nsa_pop3_connect (struct socket_s *socket);
/****/

/* imap.c */
int nsa_imap_connect (struct socket_s *socket);
/****/

/* mysql.c */
#ifdef HAVE_MYSQLLIB
int nsa_mysql_connect (const int index);
#endif  /* HAVE_MYSQLLIB */
/****/

/* ssh.c */
#ifdef HAVE_SSHLIB
int nsa_ssh_connect (const int index);
#endif /* HAVE_SSHLIB */
/****/

/* cmd.c */
int cmd_check_code (struct socket_s *socket, const char *VALUE_CODE);
int cmd_logout_session (struct socket_s *socket, const char *quit, const char *answer);
char * cmd_recv (struct socket_s *socket);
int cmd_send (struct socket_s *socket, ...);
/****/

/* xfcntl */
int xfcntl (int fd, int cmd, long arg);
/****/

#endif /* _PROTOTYPE_H_ */
