/*
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "global.h"
#include "prototype.h"
#include "typedef.h"
#include "have_config.h"

int
cmd_check_code (struct socket_s *socket, const char *VALUE_CODE)
{
  char *code = NULL;
  char *p_code = NULL;
  const char *value_code = NULL;
  int ret;

  ret = 0;
  code = cmd_recv (socket);
  if (code && strlen (code))
    {
      value_code = VALUE_CODE;
      p_code = code;
      while (*p_code && *value_code && ret != -1)
        {
          if (*p_code++ != *value_code++)
            ret = -1;
        }
      value_code = NULL;
      XFREE (&code);
    }
  else
    ret = -1;

  return ret;
}

int
cmd_logout_session (struct socket_s *socket, const char *quit, const char *answer)
{
  if (cmd_send (socket, quit) == -1)
    return err_proto_socket;

  if (cmd_check_code (socket, answer) == -1)
    return err_proto_socket;

  return 0;
}

char *
cmd_recv (struct socket_s *socket)
{
  struct timeval time;
  fd_set fd_read;
  char *buff = NULL;
  int ret, size, n_read, used;

  ret = 0;

#ifdef HAVE_SSLLIB
  if (p_opt->ssl)
    ret = SSL_pending (socket->ssl);
#endif /* HAVE_SSLLIB */

  if (ret < 1)
    {
      time.tv_sec = 7; 
      time.tv_usec = 0;

      FD_ZERO (&fd_read);
      FD_SET (socket->socket, &fd_read);
      ret = select (socket->socket+1, &fd_read, NULL, NULL, &time);
    }

  if (ret < 1)
    {
#ifdef HAVE_SSLLIB
      if (p_opt->ssl)
        ret = SSL_pending (socket->ssl);
#endif /* HAVE_SSLLIB */
      if (ret < 1)
        return NULL;
    }

  size = 500;
  used = 0;
  buff = xcalloc (size);
  while (1)
    {
#ifdef HAVE_SSLLIB
      if (p_opt->ssl)
         n_read = SSL_read (socket->ssl, buff+used, (size-used)-1);
      else
#endif /* HAVE_SSLLIB */
        n_read = recv (socket->socket, buff+used, (size-used)-1, 0);
      if (n_read < (size-used)-1)
        {
          if (n_read == -1 && debug)
            fprintf(stderr, "recv error.\n");
          break;
        }
      size += 500;
      used += n_read;
      buff = realloc (buff, size);
      memset (buff+used, 0, size-used);
    }

  return (char*)buff;
}

int
cmd_send (struct socket_s *socket, ...)
{
  va_list ap;
  char *arg = NULL;
  char *s_send = NULL;
  int size, n_send;

  size = 0;
  va_start (ap, 1);
  arg = va_arg (ap, char*);
  while (arg)
    {
      s_send = realloc (s_send, size+strlen (arg)+1);
      memcpy (s_send+size, arg, strlen (arg));
      size += strlen (arg);
      *(s_send+size) = 0;
      arg = va_arg (ap, char*);
    }
  va_end (ap);

#ifdef HAVE_SSLLIB
  if (p_opt->ssl)
    {
      n_send = SSL_write (socket->ssl, s_send, size);
      if (n_send == -1 && debug)
        fprintf(stderr, "SSL_write(), error socket, text don't recved\n");
    }
  else
    {
#endif /* HAVE_SSLLIB */
      n_send = send (socket->socket, s_send, size, 0);
      if (n_send == -1 && debug)
        perror ("[send()] ");
#ifdef HAVE_SSLLIB
    }
#endif /* HAVE_SSLLIB */

  XFREE (&s_send);
  return n_send;
}
