/*
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <string.h>
#include <stdio.h>

#include "typedef.h"
#include "global.h"
#include "prototype.h"

int
nsa_ftp_client (struct socket_s *socket, char **permission)
{
  int ret;

  ret = nsa_ftp_connect (socket);
  if (ret)
    return -1;

  if (p_opt->get_ftp_write)
    nsa_get_ftp_permission (socket, permission);

  return 0;
}

int
nsa_ftp_connect (struct socket_s *socket)
{
#ifdef HAVE_SSLLIB
  int ssl;

  ssl = p_opt->ssl;
  p_opt->ssl = 0;
#endif /* HAVE_SSLLIB */

  if (cmd_check_code (socket, FTP_ACCEUIL) == -1)
    return err_proto_acceuil;

#ifdef HAVE_SSLLIB
  if (ssl)
    {
      if (cmd_send (socket, FTP_SSL) == -1)
        return -1;

      if (cmd_check_code (socket, FTP_SSL_OK) == -1)
        return err_proto_no_ssl;

      socket->ssl = connect_ssl(socket->socket);
      if (!socket->ssl)
        return err_proto_connect_ssl;
      p_opt->ssl = ssl;
    }
#endif /* HAVE_SSLLIB */

  if (cmd_send  (socket, "USER ", p_opt->login, "\r\n", NULL) == -1)
    return err_proto_socket;

  if (cmd_check_code (socket, FTP_LOGIN_SUCCESS) == -1)
    return err_proto_login;

  if (cmd_send (socket, "PASS ", p_opt->passwd, "\r\n", NULL) == -1)
    return err_proto_socket;

  if (cmd_check_code (socket, FTP_PASSWD_SUCCESS) == -1)
    return err_proto_passwd;

  return proto_no_err;
}

void
nsa_get_ftp_permission (struct socket_s *socket_ftp, char **permission)
{
  char *buff = NULL;
  struct socket_s *socket_pasv = NULL;

  socket_pasv = nsa_ftp_creat_pasv_socket (socket_ftp);
  if (!socket_pasv)
    return;

  cmd_send (socket_ftp, FTP_CMD_LIST, NULL);
#ifdef HAVE_SSLLIB
  if (p_opt->ssl)
    {
      socket_pasv->ssl = connect_ssl (socket_pasv->socket);
      if (!socket_pasv->ssl)
        {
          free_socket (&socket_pasv);
          return;
        }
    }
#endif /* HAVE_SSLLIB */

  buff = cmd_recv (socket_pasv);
  if (buff && *(buff+10))
    {
      *permission = xcalloc (sizeof(char)*11);
      memcpy (*permission, buff, sizeof(char)*10);
    }

  free_socket (&socket_pasv);
  XFREE (&buff);
}

struct socket_s *
nsa_ftp_creat_pasv_socket (struct socket_s *socket_ftp)
{
  struct pasv_s pasv_data;
  struct socket_s *pasv = NULL;
  char *buff = NULL;

  if (cmd_send (socket_ftp, FTP_CMD_PASV, NULL) == -1)
    return NULL;

  buff = cmd_recv (socket_ftp);
  get_pasiv_data (buff, &pasv_data);
  XFREE (&buff);

  pasv = create_socket (pasv_data.ip, pasv_data.port);
  if (!pasv)
    return NULL;

  if (connect_socket (pasv) == -1)
    free_socket (&pasv);

  return pasv;
}

void
get_pasiv_data (const char *info_serv, struct pasv_s *data)
{
  int i;
  int j;
  int *p_tab = NULL;
  char *search = NULL;
  char buff[10] = {0};

  search = strchr (info_serv, '(');
  if (!search)
    return;

  memset (data, 0, sizeof(struct pasv_s));

  j = 0;
  i = 0;
  while (*search && i < 4)
    {
      search++;
      if (*search == ',')
        {
          i++;
          data->ip[j++] = '.';
        }
      else
        data->ip[j++] = *search;
    }

  j--;
  i = 0;
  data->ip[j] = 0;
  data->tab[1] = -1;
  p_tab = data->tab;
  while (*search++ && data->tab[1] == -1)
    {
      if (*search == ',' || *search == ')')
        {
          *p_tab++ = xstrtol(buff);
          memset (buff, 0, strlen(buff));
          i = 0;
        }
      else
        buff[i++] = *search;
    }
  data->port = (data->tab[0] * 256) + data->tab[1];

  p_tab = NULL;
  search = NULL;
}
