/*
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "typedef.h"
#include "global.h"
#include "prototype.h"

void *
xmalloc (const int size)
{
  void *ptr = NULL;

  if (size < 1)
    {
      if (debug)
        fprintf(stderr, "size invalid to malloc()\n");
    }
  else
    {
      ptr = malloc (size);
      if (!ptr)
        exit_program (err_mem_alloc);
    }

  return ptr;
}

void *
xcalloc (const int size)
{
  void *ptr = NULL;

  if (size < 1)
    {
      if (debug)
        fprintf (stderr, "size invalid to malloc()\n");
    }
  else
    {
      ptr = calloc (1, size);
      if (!ptr)
        exit_program (err_mem_alloc);
    }

  return ptr;
}

char *
xstrdup (const char *str)
{
  char *cpy = NULL;
  int len;

  len = strlen (str);
  cpy = xcalloc (len+1);
  strncpy (cpy, str, len);
  return cpy;
}

void
xfree (void **ptr)
{
  if (*ptr)
    {
      free (*ptr);
      *ptr = NULL;
    }
}

void
clean_memory (void)
{
  int i;

  if (th_argv)
    {
      i = 0;
      while (i < p_opt->n_thread)
        XFREE (&th_argv[i++]);
      XFREE (&th_argv);
    }
  if (p_opt)
    {
      XFREE (&p_opt->login);
      XFREE (&p_opt->passwd);
      XFREE (&p_opt->proto);
      p_opt = NULL;
    }
  if (p_stt)
    p_stt = NULL;
  free_pile ();
}