/*
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "have_config.h"
#ifdef HAVE_SSLLIB
#include <openssl/ssl.h>
#endif /* HAVE_SSLLIB */

#include <errno.h>
#include <stdio.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>

#include "global.h"
#include "typedef.h"
#include "prototype.h"

struct socket_s *
create_socket (const char *ip, const int port)
{
  struct socket_s *sock = NULL;

  sock = xmalloc (sizeof(struct socket_s));
  sock->socket = create_socket_stream ();
  if (sock->socket == -1)
    {
      XFREE (&sock);
      return NULL;
    }

  sock->to.sin_family = AF_INET;
  sock->to.sin_port = htons (port);
  sock->to.sin_addr.s_addr = inet_addr (ip);
#ifdef HAVE_SSLLIB
  sock->ssl = NULL;
#endif /* HAVE_SSLLIB */

  return sock;
}

int
create_socket_stream (void)
{
  int socket_stream;

  socket_stream = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (socket_stream == -1 && debug)
      perror ("[socket()] ");

  return socket_stream;
}

int
connect_socket (struct socket_s *sock)
{
  struct timeval time;
  int ret;
  int res;
  fd_set fd_read;
  socklen_t size;

  time.tv_sec = p_opt->timeout;
  time.tv_usec = 0;

  FD_ZERO (&fd_read);
  FD_SET (sock->socket, &fd_read);

  if (xfcntl (sock->socket, F_SETFL, O_NONBLOCK) == -1)
    return -1;

  if (connect (sock->socket, (struct sockaddr*)&sock->to, sizeof(sock->to)) == -1)
    {
      if (select (sock->socket+1, NULL, &fd_read, NULL, &time) < 1)
        return -1;
    }

  res = -1;
  size = sizeof(res);
  ret = getsockopt (sock->socket, SOL_SOCKET, SO_ERROR, &res, &size);
  if (res || ret == -1)
    {
      if (ret == -1 && debug)
        perror ("[getsockopt()] ");
      return -1;
    }

  if (xfcntl (sock->socket, F_SETFL, O_SYNC) == -1)
    return -1;

  return 0;
}

/*
 * on passe un double pointeur pour initialiser la var
 * après l'avoir free, sinon elle serait juste free
 */
void
free_socket (struct socket_s **socket)
{
  if (*socket)
    {
#ifdef HAVE_SSLLIB
      if (p_opt->ssl && (*socket)->ssl)
        free_ssl (socket);
#endif /* HAVE_SSLLIB */
      if ((*socket)->socket != -1)
        close ((*socket)->socket);
      XFREE (socket);
    }
}

#ifdef HAVE_SSLLIB
SSL *
connect_ssl (int socket)
{
  SSL *ssl = NULL;
  SSL_CTX *ctx = NULL;

  OpenSSL_add_ssl_algorithms();
  ctx = SSL_CTX_new(SSLv23_method());
  if (!ctx)
    return NULL;

  ssl = SSL_new (ctx);
  SSL_CTX_free (ctx);
  ctx = NULL;

  if (!ssl && debug)
    fprintf(stderr, "init struct ssl failed\n");
  else
    {
      SSL_set_fd (ssl, socket);
      if (SSL_connect (ssl) < 0)
        {
          if (debug)
            fprintf(stderr, "SSL_connect() connect ssl failed\n");
          SSL_free (ssl);
          ssl = NULL;
        }
    }

  return ssl;
}

void
free_ssl (struct socket_s **socket)
{
  int fd;
  
  fd = SSL_get_fd ((*socket)->ssl);
  if (fd != -1)
    close (fd);
  SSL_shutdown ((*socket)->ssl);
  SSL_free ((*socket)->ssl);
  (*socket)->ssl = NULL;
}
#endif /* HAVE_SSLLIB */
